import argparse
import os
from mapactionpy_controller.crash_move_folder import CrashMoveFolder
import mapactionpy_controller.name_convention as data_name_convention


def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
        return False
    else:
        return arg


def test_contents_of_dir(dir, name_conv_definition, file_ext):
    nc = data_name_convention.NamingConvention(name_conv_definition)
    nc.regex.groupindex
    return_code = 0

    print("*****************")
    print("CHECKING DIR {}".format(dir))
    print("*****************")

    for root, dirs, files in os.walk(dir):  # pylint: disable=unused-variable
        for f in files:
            basename, extension = os.path.splitext(f)
            if extension in file_ext:
                result = nc.validate(basename)
                if not result:
                    print("error filename does not match regex: {}".format(f))
                    return_code += 1
                elif result.is_valid:
                    pass
                    # print("valid filename: {}".format(f))
                else:
                    print("one or more clauses not found in lookup tables : {}".format(f))
                    rdict = result._asdict()
                    return_code += 1

                    for clausename in nc.regex.groupindex:
                        clause_details = rdict[clausename]
                        # print(clausename, cdict)
                        if not clause_details.is_valid:
                            print("\t{} is not a recognised value for the clause {}".format(
                                clause_details.Value,
                                clausename))

                    print

    return return_code


def main():
    args = get_args()
    cmf = CrashMoveFolder(args.cmf_config_path)
    return_code = 0

    # test data names
    return_code += test_contents_of_dir(cmf.active_data, cmf.data_nc_definition, '.shp')

    # test layer names
    return_code += test_contents_of_dir(cmf.layer_rendering, cmf.layer_nc_definition, '.lyr')

    # test mxd names
    return_code += test_contents_of_dir(cmf.mxd_products, cmf.mxd_nc_definition, '.mxd')

    # Quit with the exist code
    return return_code


def get_args():
    parser = argparse.ArgumentParser(
        description=('This tool checks the conformance with the naming-convention for selected'
                     'files within the Crash Move Folder')
    )
    parser.add_argument("cmf_config_path", help="path to layer directory", metavar="FILE",
                        type=lambda x: is_valid_file(parser, x))

    return parser.parse_args()


if __name__ == "__main__":
    result = main()
    exit(result)




class MapReport:
    """
    MapReport - Report accumulated while the Map Product is generated.
    Contains overall summary and a status for each layer.
    """

    def __init__(self, productName):
        """
        Initialise to success for the product

        Arguments:
            productName {str} -- Map product name
        """
        self.result = "Failure"
        self.productName = productName
        self.summary = "No layers provided in recipe for '" + self.productName + "' product."
        self.classification = ""  # Not used
        self.results = list()

    def add(self, mapResult):
        """
        Appends the result for a given layer to the report

        Arguments:
            mapResult {MapResult} -- result summary for a given layer
        """
        self.results.append(mapResult)
        self._updateSummary()

    def _updateSummary(self):
        """
        Maintains summary
        """
        # Update summary
        failCount = 0
        reportIter = 0
        for r in self.results:
            reportIter = reportIter + 1
            if (r.added is False):
                failCount = failCount + 1

        if (failCount == 0):
            self.result = "Success"
            self.summary = "'" + self.productName + "' product generated successfully."
        else:
            self.result = "Warning"
            self.summary = str(failCount) + " / " + str(reportIter) + \
                " layers could not be added to '" + self.productName + "' product."



import argparse
import jsonpickle
import os
import re
from mapactionpy_controller.crash_move_folder import CrashMoveFolder
from mapactionpy_controller.event import Event
from mapactionpy_controller.product_bundle_definition import MapRecipe


class DataSearch():
    def __init__(self, event):
        self.event = event
        self.cmf = CrashMoveFolder(self.event.cmf_descriptor_path)

    def update_search_with_event_details(self, recipe):
        for lyr in recipe.layers:
            output_str = lyr.search_definition.format(e=self.event)
            lyr.search_definition = output_str

        return recipe

    def update_recipe_with_datasources(self, recipe):
        for lyr in recipe.layers:
            lyr.data_source_path, lyr.data_name = self._find_data(lyr)

        return recipe

    def _find_data(self, lyr):
        found_datasources = []
        found_datanames = []
        for root, dirs, files in os.walk(self.cmf.active_data):  # pylint: disable=unused-variable
            for f in files:
                if re.match(lyr.search_definition, f):
                    found_datasources.append(
                        os.path.normpath(os.path.join(root, f)))
                    found_datanames.append(
                        os.path.splitext(os.path.basename(f))[0])

        return ';'.join(found_datasources), ';'.join(found_datanames),


def _is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
    else:
        return arg


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("-r", "--recipe-file", dest="recipe_file", required=True,
                        help="path to recipe json file.", metavar="FILE", type=lambda x: _is_valid_file(parser, x))
    parser.add_argument("-e", "--event", dest="event_path", required=True,
                        help="path the Event descriptor file.", metavar="FILE",
                        type=lambda x: _is_valid_file(parser, x))
    parser.add_argument("-o", "--output-file", dest="output_file", required=False,
                        help="(optional) path to the ouptut file. If omited the output is printed to stdout.",
                        metavar="FILE")

    return parser.parse_args()


def main():
    args = get_args()

    org_recipe = MapRecipe(args.recipe_file)

    ds = DataSearch(Event(args.event_path))
    updated_recipe = ds.update_search_with_event_details(org_recipe)
    updated_recipe = ds.update_recipe_with_datasources(updated_recipe)

    json_recipe = jsonpickle.encode(updated_recipe, unpicklable=False)

    if args.output_file is not None:
        with open(args.output_file, 'w') as f:
            f.write(json_recipe)
    else:
        print(json_recipe)


if __name__ == '__main__':
    main()



from pydoc import locate
import json
import re
from collections import namedtuple
from mapactionpy_controller.name_clause_validators import NamingClause


class NamingConvention:
    def __init__(self, nc_json_path):
        self.nc_json_path = nc_json_path
        self._clause_validation = {}

        with open(self.nc_json_path) as json_file:
            json_contents = json.load(json_file)

        self.regex = re.compile(json_contents['pattern'])

        rx_grp_list = self.regex.groupindex.keys()

        json_clause_names = set()
        for clause_def in json_contents['clauses']:
            json_clause_names.add(clause_def['name'])

        if not (set(rx_grp_list) == json_clause_names):
            raise NamingException(
                'Error in {}. Mismatch between clause definition {} '
                'and groups name in regular expresion {}')

        for clause_def in json_contents['clauses']:
            clause_name = clause_def['name']
            validator_name = clause_def['validator']

            try:
                Validator = locate(validator_name)
                # Instantiate the class (pass arguments to the constructor, if needed)
                dnlc = Validator(self.nc_json_path, **clause_def)
            except TypeError:
                raise NamingException('Error in {}. The validation type {} cannot be loaded'
                                      ''.format(self.nc_json_path, validator_name))

            if isinstance(dnlc, NamingClause):
                self._clause_validation[clause_name] = dnlc
            else:
                raise NamingException('Error in {}. The specified validator class {} is not '
                                      'an instance of mapactionpy_controller.name_convention.NameClause'
                                      ''.format(self.nc_json_path, validator_name))

    def validate(self, data_name):
        regex_res = self.regex.search(data_name)
        if regex_res:
            result = {}
            for key in self._clause_validation:
                v = self._clause_validation[key]
                result[key] = v.validate(regex_res.group(key))

            class NamingResult(namedtuple(
                    'NamingResult', self._clause_validation.keys())):
                __slots__ = ()

                @property
                def is_valid(self):
                    return all(x.is_valid for x in self._asdict().values())

            return NamingResult(**result)
        else:
            return None


class NamingException(Exception):
    pass



# flake8: noqa
recipe_without_positive_iso3_code = \
    r'''{
    "title": "{e.affectedcountry}: Overview Map",
    "layers": [
   {
       "map_frame": "Main Map",
       "layer_group": "Admin - Polygons",
       "layer_display_name": "Admin - AffectedCountry - py",
       "search_definition": "^{e.affected_country_iso3}_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(.+)shp$",
       "data_source_path": "",
       "data_name": "",
       "rendering": "Admin - AffectedCountry - py",
       "definition_query": "None",
       "visible": "Yes"
   }
    ]
}'''

recipe_without_negative_iso3_code = \
    r'''{
    "title": "{e.affectedcountry}: Overview Map",
    "layers": [
       {
       "map_frame": "Main Map",
       "layer_group": "Admin - Polygons",
       "layer_display_name": "Admin - SurroundingCountry - py",
       "search_definition": "^(?!({e.affected_country_iso3}))_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(.+)shp$",
       "data_source_path": "",
       "data_name": "",
       "rendering": "Admin - SurroundingCountry - py",
       "definition_query": "ADM0_NAME <> '[reference country]'",
       "visible": "Yes"
   }
    ]
}'''

recipe_with_positive_iso3_code = \
    r'''{
    "title": "{e.affectedcountry}: Overview Map",
    "layers": [
   {
       "map_frame": "Main Map",
       "layer_group": "Admin - Polygons",
       "layer_display_name": "Admin - AffectedCountry - py",
       "search_definition": "^moz_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(.+)shp$",
       "data_source_path": "",
       "data_name": "",
       "rendering": "Admin - AffectedCountry - py",
       "definition_query": "None",
       "visible": "Yes"
   }
    ]
}'''

recipe_with_negative_iso3_code = \
    r'''{
    "title": "{e.affectedcountry}: Overview Map",
    "layers": [
       {
       "map_frame": "Main Map",
       "layer_group": "Admin - Polygons",
       "layer_display_name": "Admin - SurroundingCountry - py",
       "search_definition": "^(?!(moz))_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(.+)shp$",
       "data_source_path": "",
       "data_name": "",
       "rendering": "Admin - SurroundingCountry - py",
       "definition_query": "ADM0_NAME <> '[reference country]'",
       "visible": "Yes"
   }
    ]
}'''

recipe_result_one_dataset_per_layer = \
    r'''{
    "title": "{e.affectedcountry}: Overview Map",
    "layers": [
   {
       "map_frame": "Main Map",
       "layer_group": "Admin - Polygons",
       "layer_display_name": "Admin - AffectedCountry - py",
       "search_definition": "^moz_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(.+)shp$",
       "data_source_path": "D:/MapAction/2019MOZ01/GIS/2_Active_Data/202_admn/moz_admn_ad0_py_s0_unknown_pp.shp",
       "data_name": "moz_admn_ad0_py_s0_unknown_pp",
       "rendering": "Admin - AffectedCountry - py",
       "definition_query": "None",
       "visible": "Yes"
   }
    ]
}'''

fixture_datasource_query = r"""
{
"settlement_points": "^{e.affected_country_iso3}_stle_stl_pt_(.*?)_(.*?)_([phm][phm])(_(.+))",
"surrounding_counties": "^(?!({e.affected_country_iso3}))_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(_(.+))"
"airports_points": "D:\MapAction\2019-06-12-GBR\GIS\2_Active_Data\232_tran\scr_tran_air_pt_s1_ourairports_pp.shp"
}
"""

fixture_datasource_intermediatory_query = r"""
{
"settlement_points": "^gbr_stle_stl_pt_(.*?)_(.*?)_([phm][phm])(_(.+))",
"surrounding_counties": "^(?!(gbr))_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(_(.+))"
"airports_points": "D:\MapAction\2019-06-12-GBR\GIS\2_Active_Data\232_tran\scr_tran_air_pt_s1_ourairports_pp.shp"
}
"""

fixture_datasource_result_one_dataset_per_layer = r"""
{
"settlement_points": "D:\MapAction\2019-06-12-GBR\GIS\2_Active_Data\gbr_stle_stle_pt_s0_naturalearth_pp.shp",
"airports_points": "D:\MapAction\2019-06-12-GBR\GIS\2_Active_Data\232_tran\scr_tran_air_pt_s1_ourairports_pp.shp"
}
"""

fixture_datasource_result_missing_layer = r"""
{
"settlement_points": "",
"airports_points": "D:\MapAction\2019-06-12-GBR\GIS\2_Active_Data\232_tran\scr_tran_air_pt_s1_ourairports_pp.shp"
}
"""

walk_single_admn_file_search_search = \
    [
        (
            "D:/MapAction/2019MOZ01/GIS/2_Active_Data",
            ['202_admn'],
            ['desktop.ini']
        ),
        (
            "D:/MapAction/2019MOZ01/GIS/2_Active_Data/202_admn",
            [],
            ['moz_admn_ad0_ln_s0_unknown_pp.CPG',
             'moz_admn_ad0_ln_s0_unknown_pp.dbf',
             'moz_admn_ad0_py_s0_unknown_pp.sbx',
             'moz_admn_ad0_py_s0_unknown_pp.sbn',
             'moz_admn_ad0_py_s0_unknown_pp.prj',
             'moz_admn_ad0_py_s0_unknown_pp.dbf',
             'moz_admn_ad0_ln_s0_unknown_pp.shp',
             'moz_admn_ad0_ln_s0_unknown_pp.shx',
             'moz_admn_ad0_py_s0_unknown_pp.CPG',
             'moz_admn_ad0_ln_s0_unknown_pp.sbx',
             'moz_admn_ad0_ln_s0_unknown_pp.prj',
             'moz_admn_ad0_ln_s0_unknown_pp.sbn',
             'moz_admn_ad1_py_s1_mapaction_pp.shx',
             'moz_admn_ad1_ln_s1_mapaction_pp.shp.xml',
             'moz_admn_ad1_ln_s1_mapaction_pp.prj',
             'moz_admn_ad1_ln_s1_mapaction_pp.sbx',
             'moz_admn_ad1_ln_s1_mapaction_pp.sbn',
             'moz_admn_ad1_ln_s1_mapaction_pp.shp.LAPTOP-F7PICP7J.8304.13164.sr.lock',
             'moz_admn_ad1_py_s1_mapaction_pp.shp.xml',
             'moz_admn_ad1_py_s1_mapaction_pp.sbx',
             'moz_admn_ad1_py_s1_mapaction_pp.sbn',
             'moz_admn_ad1_py_s1_mapaction_pp.shp.LAPTOP-F7PICP7J.8304.13164.sr.lock',
             'moz_admn_ad1_py_s1_mapaction_pp.prj',
             'moz_admn_ad1_ln_s1_mapaction_pp.shx',
             'moz_admn_ad1_py_s1_mapaction_pp.dbf',
             'moz_admn_ad1_ln_s1_mapaction_pp.dbf',
             'moz_admn_ad0_py_s0_unknown_pp.shx',
             'moz_admn_ad1_ln_s1_mapaction_pp.CPG',
             'moz_admn_ad0_py_s0_unknown_pp.shp',
             'moz_admn_ad0_py_s0_unknown_pp.shp.xml',
             'moz_admn_ad1_py_s1_mapaction_pp.CPG',
             'moz_admn_ad1_py_s1_mapaction_pp.shp',
             'moz_admn_ad1_ln_s1_mapaction_pp.shp',
             'desktop.ini']
        )
    ]



# TODO: asmith 2020/03/04
#
# Compare this class to the implementation here:
#       cmf_watcher.cmf_handler._hash_files(...)
# I think the implenmentation here (ie in the DataSource class) handles shapefiles better - notably
# handling lock files. However the use case in the cmf_watcher module is different and creating a
# new DataSource object each time might not be appropriate. Is it possible to refactor this so that it
# achieves boths use-cases?
import os
import hashlib
import glob


class DataSource:
    def __init__(self, pathToDataSource):
        self.parts = list()
        self.pathToDataSource = pathToDataSource

    def calculate_checksum(self):
        if (os.path.isfile(self.pathToDataSource)):
            self.parts = self.constituentParts(self.pathToDataSource)
        elif (os.path.isdir(self.pathToDataSource)):
            self.parts = self.directoryContents(self.pathToDataSource)

        hash = hashlib.md5()
        for fn in self.parts:
            if os.path.isfile(fn):
                hash.update(open(fn, "rb").read())
        return hash.hexdigest()

    def constituentParts(self, fileName):
        paths_to_hash = list()
        directoryName = os.path.dirname(fileName)
        baseName = os.path.basename(fileName)
        root, ext = os.path.splitext(baseName)
        filter = os.path.join(directoryName, root) + "*"
        files = glob.glob(filter)

        for file in files:
            root, ext = os.path.splitext(file)
            if (not(ext.endswith(".lock"))):
                paths_to_hash.append(file)
        return paths_to_hash

    def directoryContents(self, file_directory):
        file_directory = os.path.abspath(file_directory)
        paths_to_hash = list()

        for currentpath, folders, files in os.walk(file_directory):
            for file in files:
                paths_to_hash.append(os.path.join(currentpath, file))
        return(paths_to_hash)



from datetime import datetime


# TODO: asmith 2020/03/06
#
# 1) Based on the description in the docstring, would "LayerResult" be a more appropriate name for
# this class?
#
# 2) What are valid states for this object. eg Can I legitimately have:
#       * `self.added = False` and `self.message = "" `
#       * `self.added = True` and `self.hash = ""`
#       * Does the `self.hash` value necessarily corespond to the `self.dataSource`?
# I'd suggest that some methods to ensure consitancy, rather than rely on the caller to ensure this.
# Would it be appropriate to use the DataSource class here?
class MapResult:
    """
    MapResult - result for adding a layer to the Map Product
    """

    def __init__(self, layerName):
        """
        Constructor, initialises new Map Result for the layer

        Arguments:
           layerName {str} -- name of the map layer being added
        """
        # TODO: asmith 2020/03/06
        # Ideally stick to all lower_case_variable_names
        # https://www.python.org/dev/peps/pep-0008/#method-names-and-instance-variables
        self.layerName = layerName
        self.dataSource = ""
        now = datetime.now()
        # dd/mm/YY H:M:S
        self.dateStamp = now.strftime("%d/%m/%Y %H:%M:%S")
        self.added = False
        self.message = ""
        self.hash = ""



import os
from unittest import TestCase
import fixtures
from mapactionpy_controller.product_bundle_definition import MapRecipe
from mapactionpy_controller.crash_move_folder import CrashMoveFolder
from mapactionpy_controller.data_search import DataSearch
from mapactionpy_controller.event import Event
import jsonpickle
import six
# works differently for python 2.7 and python 3.x
try:
    from unittest import mock
except ImportError:
    import mock


class TestMAController(TestCase):

    def setUp(self):
        self.parent_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

        self.recipe_descriptor_path = os.path.join(self.parent_dir, 'example', 'product_bundle_example.json')
        self.recipe = MapRecipe(self.recipe_descriptor_path)

        self.cmf_descriptor_path = os.path.join(self.parent_dir, 'example', 'cmf_description.json')
        # self.cmf = CrashMoveFolder(self.cmf_descriptor_path, verify_on_creation=False)
        self.event_descriptor_path = os.path.join(self.parent_dir, 'example', 'event_description.json')
        self.event = Event(self.event_descriptor_path)

    def test_equality_of_map_recipes(self):
        test_recipe1 = MapRecipe(None, fixtures.recipe_with_positive_iso3_code)
        test_recipe2 = MapRecipe(None, fixtures.recipe_with_positive_iso3_code)

        self.assertEqual(test_recipe1, test_recipe2)
        test_recipe1.title = "something different"
        self.assertNotEqual(test_recipe1, test_recipe2)

        test_recipe3 = MapRecipe(None, fixtures.recipe_with_positive_iso3_code)
        self.assertEqual(test_recipe2, test_recipe3)
        test_recipe3.layers.pop()
        self.assertNotEqual(test_recipe2, test_recipe3)

    def test_serialise_and_deserialise_map_recipe(self):
        recipes_fixtures = [
            fixtures.recipe_with_positive_iso3_code,
            fixtures.recipe_result_one_dataset_per_layer,
            fixtures.recipe_without_positive_iso3_code
        ]

        for fixture_str in recipes_fixtures:
            test_recipe = MapRecipe(None, str_def=fixture_str)

            self.assertEqual(test_recipe, jsonpickle.decode(jsonpickle.encode(test_recipe)))
            self.assertEqual(test_recipe, MapRecipe(None, str_def=jsonpickle.encode(test_recipe, unpicklable=False)))
            self.assertEqual(test_recipe, MapRecipe(None, str_def=jsonpickle.encode(test_recipe)))
            self.assertNotEqual(test_recipe, MapRecipe(None, str_def=fixtures.recipe_with_negative_iso3_code))

    def test_substitute_iso3_in_regex(self):
        ds = DataSearch(self.event)

        reference_recipe = MapRecipe(
            None, str_def=fixtures.recipe_with_positive_iso3_code)
        pos_recipe = MapRecipe(
            None, str_def=fixtures.recipe_without_positive_iso3_code)
        updated_pos_recipe = ds.update_search_with_event_details(pos_recipe)
        self.assertEqual(updated_pos_recipe, reference_recipe)

        reference_recipe = MapRecipe(
            None, str_def=fixtures.recipe_with_negative_iso3_code)
        neg_recipe = MapRecipe(
            None, str_def=fixtures.recipe_without_negative_iso3_code)
        updated_neg_recipe = ds.update_search_with_event_details(neg_recipe)
        self.assertEqual(updated_neg_recipe, reference_recipe)

    @mock.patch('mapactionpy_controller.data_search.os.path')
    @mock.patch('mapactionpy_controller.data_search.os')
    def test_search_for_shapefiles(self, mock_os, mock_path):
        ds = DataSearch(self.event)

        # case where there is exactly one dataset per query
        mock_os.walk.return_value = fixtures.walk_single_admn_file_search_search
        mock_path.join.return_value = \
            r'D:/MapAction/2019MOZ01/GIS/2_Active_Data/202_admn/moz_admn_ad0_py_s0_unknown_pp.shp'
        mock_path.normpath.return_value = \
            r'D:/MapAction/2019MOZ01/GIS/2_Active_Data/202_admn/moz_admn_ad0_py_s0_unknown_pp.shp'
        mock_path.splitext.return_value = (r'moz_admn_ad0_py_s0_unknown_pp', r'.shp')
        reference_recipe = MapRecipe(
            None, str_def=fixtures.recipe_result_one_dataset_per_layer)
        test_recipe = MapRecipe(None, str_def=fixtures.recipe_with_positive_iso3_code)
        self.assertNotEqual(test_recipe, reference_recipe)
        updated_test_recipe = ds.update_recipe_with_datasources(test_recipe)

        self.assertEqual(updated_test_recipe, reference_recipe)

    def test_cmf_path_validation(self):

        cmf_partial_fail = os.path.join(
            self.parent_dir, 'tests', 'testfiles', 'fixture_cmf_description_one_file_and_one_dir_not_valid.json')

        # test validation on creation (for failing case) using default parameters
        self.assertRaises(ValueError, CrashMoveFolder, self.cmf_descriptor_path)
        # force validation on creation, for a json file we know would otherwise fail:
        self.assertRaises(ValueError, CrashMoveFolder, self.cmf_descriptor_path, verify_on_creation=True)
        # test validation is correctly disabled on creation, for a json file we know would otherwise fail:
        test_cmf = CrashMoveFolder(self.cmf_descriptor_path, verify_on_creation=False)
        self.assertIsInstance(test_cmf, CrashMoveFolder)

        # check message included in the ValueError:
        with self.assertRaises(ValueError) as cm:
            test_cmf = CrashMoveFolder(cmf_partial_fail, verify_on_creation=True)

        if six.PY2:
            self.assertRegexpMatches(str(cm.exception), "mxd_templates")
            self.assertNotRegexpMatches(str(cm.exception), "original_data")
        else:
            self.assertRegex(str(cm.exception), "mxd_templates")
            self.assertNotRegex(str(cm.exception), "original_data")

        # create a valid CMF object and then test paths, after creation
        test_cmf_path = os.path.join(self.parent_dir, 'example', 'cmf_description_flat_test.json')
        test_cmf = CrashMoveFolder(test_cmf_path)
        self.assertTrue(test_cmf.verify_paths())
        test_cmf_path = os.path.join(self.parent_dir, 'example', 'cmf_description_relative_paths_test.json')
        test_cmf = CrashMoveFolder(test_cmf_path)
        self.assertTrue(test_cmf.verify_paths())
        test_cmf.active_data = os.path.join(self.parent_dir, 'DOES-NOT-EXIST')
        self.assertFalse(test_cmf.verify_paths())




class LabelClass:
    """
    Enables selection of properties to support labels in a Layer
    """

    def __init__(self, row):
        self.className = row["className"]
        self.expression = row["expression"]
        self.SQLQuery = row["SQLQuery"]
        self.showClassLabels = row["showClassLabels"]



import importlib
import subprocess
from setuptools import setup, find_packages
from os import path, environ

_base_version = '0.12'

root_dir = path.abspath(path.dirname(__file__))


def readme():
    with open(path.join(root_dir, 'README.md')) as f:
        return f.read()


# See https://packaging.python.org/guides/single-sourcing-package-version/
# This uses method 4 on this list combined with other methods.
def _get_version_number():
    travis_build = environ.get('TRAVIS_BUILD_NUMBER')
    travis_tag = environ.get('TRAVIS_TAG')

    if travis_build:
        if travis_tag:
            version = travis_tag
        else:
            version = '{}.dev{}'.format(_base_version, travis_build)

        with open(path.join(root_dir, 'VERSION'), 'w') as version_file:
            version_file.write(version.strip())
    else:
        try:
            ver = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'])
            version = '{}+local.{}'.format(_base_version, ver.decode('ascii').strip())
        except Exception:
            with open(path.join(root_dir, 'VERSION')) as version_file:
                version = version_file.read().strip()

    return version


def get_install_requires():
    dependancies = [
        'jsonpickle',
        'pycountry',
        'six'
    ]
    dependancies.append(get_gis_environment())

    return dependancies


def get_gis_environment():
    gis_dependancies = []

    # The key is the module name that is detected
    # The value is the module name that will be installed if the key is detected.
    possible_envs = {
        'arcpy': 'mapactionpy_arcmap',
        'qgis.core': 'mapactionpy_qgis'
    }

    for env_mod, target_mod in possible_envs.items():
        try:
            importlib.import_module(env_mod)
            gis_dependancies.append(target_mod)
        except ImportError:
            pass

    return gis_dependancies


setup(name='mapactionpy_controller',
      version=_get_version_number(),
      description='Controls the workflow of map and infographic production',
      long_description=readme(),
      long_description_content_type="text/markdown",
      url='http://github.com/mapaction/mapactionpy_controller',
      author='MapAction',
      author_email='github@mapaction.com',
      license='GPL3',
      packages=find_packages(),
      install_requires=get_install_requires(),
      extras_require={
          'mapactionpy_arcmap': ['arcpy'],
          'mapactionpy_qgis': ['qgis.core']
      },
      test_suite='unittest',
      tests_require=['unittest'],
      zip_safe=False,
      classifiers=[
          "Development Status :: 2 - Pre-Alpha",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
          "Operating System :: OS Independent",
      ])
